
-- Source: start.sql
    
create extension dblink;


-- Source: tables/common/Account.sql
    
CREATE TABLE Account (
    mgm_id                    NUMERIC NOT NULL,
    account_id                NUMERIC NOT NULL,
    username                  VARCHAR(64),
    organization              VARCHAR(128),
    last_name                 VARCHAR(128),
    first_name                VARCHAR(128),
    position                  VARCHAR(128),
    email                     VARCHAR(128),
    creation_time             TIMESTAMPTZ,
    last_login_time           TIMESTAMPTZ,
    status                    VARCHAR(32),
    md5_encryption            BOOLEAN,
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Account
  ADD CONSTRAINT Account_pk PRIMARY KEY (mgm_id, account_id);

-- Source: tables/common/AccountGroup.sql
    
CREATE TABLE AccountGroup (
    mgm_id                      NUMERIC NOT NULL,
    account_id                  NUMERIC NOT NULL,
    account_group_id            NUMERIC NOT NULL,
    username                    VARCHAR(64),
    account_group_name          VARCHAR(64),
    account_group_type_id       NUMERIC,
    account_group_type_name     VARCHAR(64),
    account_group_type_label    VARCHAR(64),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE AccountGroup
  ADD CONSTRAINT AccountGroup_pk PRIMARY KEY (mgm_id, account_id, account_group_id);

-- Source: tables/common/Channel.sql
    
CREATE TABLE Channel
(
    mgm_id                    NUMERIC NOT NULL,
    channel_id                NUMERIC NOT NULL,
    name                      VARCHAR(256),
    label                     VARCHAR(128),
    type                      VARCHAR(50),
    arch                      VARCHAR(64),
    checksum_type             VARCHAR(32),
    summary                   VARCHAR(500),
    description               VARCHAR(4000),
    parent_channel_label      VARCHAR(128),
    original_channel_id       NUMERIC,
    organization              VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Channel
  ADD CONSTRAINT Channel_pk PRIMARY KEY (mgm_id, channel_id);

-- Source: tables/common/ChannelErrata.sql
    
CREATE TABLE ChannelErrata
(
    mgm_id                    NUMERIC NOT NULL,
    channel_id                NUMERIC NOT NULL,
    errata_id                 NUMERIC NOT NULL,
    channel_label             VARCHAR(128),
    advisory_name             VARCHAR(100),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE ChannelErrata
  ADD CONSTRAINT ChannelErrata_pk PRIMARY KEY (mgm_id, channel_id, errata_id);

-- Source: tables/common/ChannelPackage.sql
    
CREATE TABLE ChannelPackage
(
    mgm_id                    NUMERIC NOT NULL,
    channel_id                NUMERIC NOT NULL,
    package_id                NUMERIC NOT NULL,
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE ChannelPackage
  ADD CONSTRAINT ChannelPackage_pk PRIMARY KEY (mgm_id, channel_id, package_id);

-- Source: tables/common/ChannelRepository.sql
    
CREATE TABLE ChannelRepository
(
    mgm_id                    NUMERIC NOT NULL,
    channel_id                NUMERIC NOT NULL,
    repository_id             NUMERIC NOT NULL,
    repository_label          VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE ChannelRepository
  ADD CONSTRAINT ChannelRepository_pk PRIMARY KEY (mgm_id, channel_id, repository_id);

-- Source: tables/common/Errata.sql
    
CREATE TABLE Errata
(
    mgm_id                        NUMERIC NOT NULL,
    errata_id                     NUMERIC NOT NULL,
    advisory_name                 VARCHAR(100),
    advisory_type                 VARCHAR(32),
    advisory_status               VARCHAR(32),
    issue_date                    TIMESTAMPTZ,
    update_date                   TIMESTAMPTZ,
    severity                      VARCHAR(64),
    reboot_required               BOOLEAN NOT NULL DEFAULT FALSE,
    affects_package_manager       BOOLEAN NOT NULL DEFAULT FALSE,
    cve                           VARCHAR(4000),
    synopsis                      VARCHAR(4000),
    organization                  VARCHAR(128),
    synced_date                   TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Errata
  ADD CONSTRAINT Errata_pk PRIMARY KEY (mgm_id, errata_id);

-- Source: tables/common/Package.sql
    
CREATE TABLE Package
(
    mgm_id              NUMERIC NOT NULL,
    package_id          NUMERIC NOT NULL,
    name                VARCHAR(256),
    epoch               VARCHAR(16),
    version             VARCHAR(512),
    release             VARCHAR(512),
    arch                VARCHAR(64),
    type                VARCHAR(10),
    package_size        NUMERIC,
    payload_size        NUMERIC,
    installed_size      NUMERIC,
    vendor              VARCHAR(64),
    organization        VARCHAR(128),
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Package
  ADD CONSTRAINT Package_pk PRIMARY KEY (mgm_id, package_id);

-- Source: tables/common/Repository.sql
    
CREATE TABLE Repository
(
    mgm_id                    NUMERIC NOT NULL,
    repository_id             NUMERIC NOT NULL,
    label                     VARCHAR(128),
    url                       VARCHAR(2048),
    type                      VARCHAR(32),
    metadata_signed           BOOLEAN,
    organization              VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Repository
  ADD CONSTRAINT Repository_pk PRIMARY KEY (mgm_id, repository_id);

-- Source: tables/common/System.sql
    
CREATE TABLE System
(
    mgm_id                        NUMERIC NOT NULL,
    system_id                     NUMERIC NOT NULL,
    profile_name                  VARCHAR(128),
    hostname                      VARCHAR(128),
    minion_id                     VARCHAR(256),
    minion_os_family              VARCHAR(32),
    minion_kernel_live_version    VARCHAR(255),
    machine_id                    VARCHAR(256),
    registered_by                 VARCHAR(64),
    registration_time             TIMESTAMPTZ,
    last_checkin_time             TIMESTAMPTZ,
    kernel_version                VARCHAR(64),
    architecture                  VARCHAR(64),
    is_proxy                      BOOLEAN NOT NULL DEFAULT FALSE,
    proxy_system_id               NUMERIC,
    is_mgr_server                 BOOLEAN NOT NULL DEFAULT FALSE,
    organization                  VARCHAR(128),
    hardware                      TEXT,
    machine                       VARCHAR(64),
    rack                          VARCHAR(64),
    room                          VARCHAR(32),
    building                      VARCHAR(128),
    address1                      VARCHAR(128),
    address2                      VARCHAR(128),
    city                          VARCHAR(128),
    state                         VARCHAR(60),
    country                       VARCHAR(2),
    synced_date                   TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE System
  ADD CONSTRAINT System_pk PRIMARY KEY (mgm_id, system_id);

CREATE INDEX System_profile_name_idx
  ON System (profile_name);

-- Source: tables/common/SystemAction.sql
    
CREATE TABLE SystemAction
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    action_id           NUMERIC NOT NULL,
    hostname            VARCHAR(128),
    scheduler_id        NUMERIC,
    scheduler_username  VARCHAR(64),
    earliest_action     TIMESTAMPTZ,
    archived            BOOLEAN,
    pickup_time         TIMESTAMPTZ,
    completion_time     TIMESTAMPTZ,
    action_name         VARCHAR(128),
    status              VARCHAR(16),
    event               VARCHAR(100),
    event_data          TEXT,
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemAction
  ADD CONSTRAINT SystemAction_pk PRIMARY KEY (mgm_id, system_id, action_id);

-- Source: tables/common/SystemChannel.sql
    
CREATE TABLE SystemChannel
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    channel_id                  NUMERIC NOT NULL,
    name                        VARCHAR(256),
    description                 VARCHAR(4000),
    architecture_name           VARCHAR(64),
    parent_channel_id           NUMERIC,
    parent_channel_name         VARCHAR(256),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemChannel
  ADD CONSTRAINT SystemChannel_pk PRIMARY KEY (mgm_id, system_id, channel_id);

-- Source: tables/common/SystemConfigChannel.sql
    
CREATE TABLE SystemConfigChannel
(
    mgm_id                  NUMERIC NOT NULL,
    system_id               NUMERIC NOT NULL,
    config_channel_id       NUMERIC NOT NULL,
    name                    VARCHAR(128),
    position                NUMERIC,
    synced_date             TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemConfigChannel
  ADD CONSTRAINT SystemConfigChannel_pk PRIMARY KEY (mgm_id, system_id, config_channel_id);

-- Source: tables/common/SystemCustomInfo.sql
    
CREATE TABLE SystemCustomInfo
(
    mgm_id                  NUMERIC NOT NULL,
    system_id               NUMERIC NOT NULL,
    organization            VARCHAR(128) NOT NULL,
    key                     VARCHAR(64),
    description             VARCHAR(4000),
    value                   VARCHAR(4000),
    synced_date             TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemCustomInfo
  ADD CONSTRAINT SystemCustomInfo_pk PRIMARY KEY (mgm_id, organization, system_id, key);

-- Source: tables/common/SystemEntitlement.sql
    
CREATE TABLE SystemEntitlement
(
    mgm_id                  NUMERIC NOT NULL,
    system_id               NUMERIC NOT NULL,
    system_group_id         NUMERIC NOT NULL,
    name                    VARCHAR(64),
    description             VARCHAR(1024),
    group_type              NUMERIC,
    group_type_name         VARCHAR(64),
    current_members         NUMERIC,
    organization            VARCHAR(128),
    synced_date             TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemEntitlement
  ADD CONSTRAINT SystemEntitlement_pk PRIMARY KEY (mgm_id, system_id, system_group_id);

-- Source: tables/common/SystemErrata.sql
    
CREATE TABLE SystemErrata
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    errata_id           NUMERIC NOT NULL,
    hostname            VARCHAR(128),
    advisory_name       VARCHAR(100),
    advisory_type       VARCHAR(32),
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemErrata
  ADD CONSTRAINT SystemErrata_pk PRIMARY KEY (mgm_id, system_id, errata_id);

-- Source: tables/common/SystemGroup.sql
    
CREATE TABLE SystemGroup
(
    mgm_id                    NUMERIC NOT NULL,
    system_group_id           NUMERIC NOT NULL,
    name                      VARCHAR(64),
    description               VARCHAR(1024),
    current_members           NUMERIC,
    organization              VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemGroup
  ADD CONSTRAINT SystemGroup_pk PRIMARY KEY (mgm_id, system_group_id);

-- Source: tables/common/SystemGroupMember.sql
    
CREATE TABLE SystemGroupMember
(
    mgm_id                    NUMERIC NOT NULL,
    system_id                 NUMERIC NOT NULL,
    system_group_id           NUMERIC NOT NULL,
    group_name                VARCHAR(64),
    system_name               VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemGroupMember
  ADD CONSTRAINT SystemGroupMember_pk PRIMARY KEY (mgm_id, system_id, system_group_id);

-- Source: tables/common/SystemGroupPermission.sql
    
CREATE TABLE SystemGroupPermission
(
    mgm_id                    NUMERIC NOT NULL,
    system_group_id           NUMERIC NOT NULL,
    account_id                NUMERIC NOT NULL,
    group_name                VARCHAR(64),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemGroupPermission
  ADD CONSTRAINT SystemGroupPermission_pk PRIMARY KEY (mgm_id, system_group_id, account_id);

-- Source: tables/common/SystemHistory.sql
    
CREATE TABLE SystemHistory
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    history_id          NUMERIC NOT NULL,
    hostname            VARCHAR(128),
    event               VARCHAR(100),
    event_data          TEXT,
    event_time          TIMESTAMPTZ,
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemHistory
  ADD CONSTRAINT SystemHistory_pk PRIMARY KEY (mgm_id, system_id, history_id);

-- Source: tables/common/SystemNetAddressV4.sql
    
CREATE TABLE SystemNetAddressV4
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    interface_id                NUMERIC NOT NULL,
    address                     VARCHAR(64) NOT NULL,
    netmask                     VARCHAR(64),
    broadcast                   VARCHAR(64),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemNetAddressV4
  ADD CONSTRAINT SystemNetAddressV4_pk PRIMARY KEY (mgm_id, system_id, interface_id, address);

-- Source: tables/common/SystemNetAddressV6.sql
    
CREATE TABLE SystemNetAddressV6
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    interface_id                NUMERIC NOT NULL,
    scope                       VARCHAR(64) NOT NULL,
    address                     VARCHAR(64) NOT NULL,
    netmask                     VARCHAR(64),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemNetAddressV6
  ADD CONSTRAINT SystemNetAddressV6_pk PRIMARY KEY (mgm_id, system_id, interface_id, scope, address);

-- Source: tables/common/SystemNetInterface.sql
    
CREATE TABLE SystemNetInterface
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    interface_id                NUMERIC NOT NULL,
    name                        VARCHAR(32),
    hardware_address            VARCHAR(96),
    module                      VARCHAR(128),
    primary_interface           BOOLEAN NOT NULL DEFAULT FALSE,
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemNetInterface
  ADD CONSTRAINT SystemNetInterface_pk PRIMARY KEY (mgm_id, system_id, interface_id);

-- Source: tables/common/SystemOutdated.sql
    
CREATE TABLE SystemOutdated
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    packages_out_of_date        BIGINT,
    errata_out_of_date          BIGINT,
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemOutdated
  ADD CONSTRAINT SystemOutdated_pk PRIMARY KEY (mgm_id, system_id);

-- Source: tables/common/SystemPackageInstalled.sql
    
CREATE TABLE SystemPackageInstalled
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    name                VARCHAR(256),
    epoch               VARCHAR(16),
    version             VARCHAR(512),
    release             VARCHAR(512),
    arch                VARCHAR(64),
    type                VARCHAR(10),
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

CREATE INDEX IF NOT EXISTS systempackageinstalled_order_idx
  on SystemPackageInstalled (mgm_id, system_id, name, epoch, version, release, arch, type);

-- Source: tables/common/SystemPackageUpdate.sql
    
CREATE TABLE SystemPackageUpdate
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    package_id          NUMERIC NOT NULL,
    name                VARCHAR(256),
    epoch               VARCHAR(16),
    version             VARCHAR(512),
    release             VARCHAR(512),
    arch                VARCHAR(64),
    type                VARCHAR(10),
    is_latest           BOOLEAN NOT NULL DEFAULT FALSE,
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemPackageUpdate
  ADD CONSTRAINT SystemPackageUpdate_pk PRIMARY KEY (mgm_id, system_id, package_id);


-- Source: tables/common/SystemVirtualData.sql
    
CREATE TABLE SystemVirtualData
(
    mgm_id                      NUMERIC NOT NULL,
    instance_id                 NUMERIC NOT NULL,
    host_system_id              NUMERIC,
    virtual_system_id           NUMERIC,
    name                        VARCHAR(128),
    instance_type_name          VARCHAR(128),
    vcpus                       NUMERIC,
    memory_size                 NUMERIC,
    uuid                        VARCHAR(128),
    confirmed                   NUMERIC(1,0),
    state_name                  VARCHAR(128),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemVirtualData
  ADD CONSTRAINT SystemVirtualdata_pk PRIMARY KEY (mgm_id, instance_id);

-- Source: tables/common/VersionInfo.sql
    
CREATE TABLE VersionInfo
(
    name      VARCHAR(256) NOT NULL,
    label     VARCHAR(64)  NOT NULL,
    version   VARCHAR(512) NOT NULL,
    release   VARCHAR(512) NOT NULL,
    created   TIMESTAMPTZ DEFAULT (current_timestamp) NOT NULL,
    modified  TIMESTAMPTZ DEFAULT (current_timestamp) NOT NULL
);

CREATE UNIQUE INDEX versioninfo_name_label_uq
    ON VersionInfo (name, label);

-- Source: tables/common/XccdScan.sql
    
CREATE TABLE XccdScan
(
    mgm_id              NUMERIC NOT NULL,
    scan_id             NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    action_id           NUMERIC NOT NULL,
    name                VARCHAR(120),
    benchmark           VARCHAR(120),
    benchmark_version   VARCHAR(80),
    profile             VARCHAR(120),
    profile_title       VARCHAR(120),
    end_time            TIMESTAMPTZ,
    pass                NUMERIC,
    fail                NUMERIC,
    error               NUMERIC,
    not_selected        NUMERIC,
    informational       NUMERIC,
    other               NUMERIC,
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE XccdScan
  ADD CONSTRAINT XccdScan_pk PRIMARY KEY (mgm_id, scan_id);

-- Source: tables/common/XccdScanResult.sql
    
CREATE TABLE XccdScanResult
(
    mgm_id              NUMERIC NOT NULL,
    scan_id             NUMERIC NOT NULL,
    rule_id             NUMERIC NOT NULL,
    ident_id            NUMERIC NOT NULL,
    idref               VARCHAR(255),
    rulesystem          VARCHAR(80),
    system_id           NUMERIC NOT NULL,
    ident               VARCHAR(255),
    result              VARCHAR(16),
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE XccdScanResult
  ADD CONSTRAINT XccdScanResult_pk PRIMARY KEY (mgm_id, scan_id, rule_id, ident_id);

-- Source: tables/common/dual.sql
    

create table dual ( dummy char );

insert into dual values ( 'X' );

create or replace rule deny_insert_dual as on insert to dual do instead nothing;
create or replace rule deny_update_dual as on update to dual do instead nothing;
create or replace rule deny_delete_dual as on delete to dual do instead nothing;

-- Source: packages/rpm.pks
    
create schema rpm;


-- Source: packages/rpm.pkb
    
update pg_settings set setting = 'rpm,' || setting where name = 'search_path';

create or replace function isdigit(ch CHAR)
    RETURNS BOOLEAN as $$
    BEGIN
        if ascii(ch) between ascii('0') and ascii('9')
        then
            return TRUE;
        end if;
        return FALSE;
    END ;
$$ language 'plpgsql';

    
    create or replace FUNCTION isalpha(ch CHAR)
    RETURNS BOOLEAN as $$
    BEGIN
        if ascii(ch) between ascii('a') and ascii('z') or 
            ascii(ch) between ascii('A') and ascii('Z')
        then
            return TRUE;
        end if;
        return FALSE;
    END;
$$ language 'plpgsql';


    create or replace FUNCTION isalphanum(ch CHAR)
    RETURNS BOOLEAN as $$ 
    BEGIN
        if ascii(ch) between ascii('a') and ascii('z') or 
            ascii(ch) between ascii('A') and ascii('Z') or
            ascii(ch) between ascii('0') and ascii('9')
        then
            return TRUE;
        end if;
        return FALSE;
    END;
    $$ language 'plpgsql';


    create or replace FUNCTION rpmstrcmp (string1 IN VARCHAR, string2 IN VARCHAR)
    RETURNS INTEGER as $$
    declare
        str1 VARCHAR := string1;
        str2 VARCHAR := string2;
        digits VARCHAR(10) := '0123456789';
        lc_alpha VARCHAR(27) := 'abcdefghijklmnopqrstuvwxyz';
        uc_alpha VARCHAR(27) := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        alpha VARCHAR(54) := lc_alpha || uc_alpha;
        one VARCHAR;
        two VARCHAR;
        isnum BOOLEAN;
    BEGIN
        if str1 is NULL or str2 is NULL
        then
            RAISE EXCEPTION 'VALUE_ERROR.';
        end if;
      
        if str1 = str2
        then
            return 0;
        end if;
        one := str1;
        two := str2;

        <<segment_loop>>
        while one <> '' or two <> ''
        loop
            declare
                segm1 VARCHAR;
                segm2 VARCHAR;
                onechar CHAR(1);
                twochar CHAR(1);
            begin
                --raise notice 'Params: %, %',  one, two;
                -- Throw out all non-alphanum characters
                onechar := substr(one, 1, 1);
                twochar := substr(two, 1, 1);
                while one <> '' and not rpm.isalphanum(one) and onechar != '~' and onechar != '^'
                loop
                    one := substr(one, 2);
                end loop;
                while two <> '' and not rpm.isalphanum(two) and twochar != '~' and twochar != '^'
                loop
                    two := substr(two, 2);
                end loop;
                --raise notice 'new params: %, %', one, two;

                onechar := substr(one, 1, 1);
                twochar := substr(two, 1, 1);
                --raise notice 'new chars 1: %, %', onechar, twochar;
                /* handle the tilde separator, it sorts before everything else */
                if (onechar = '~' or twochar = '~')
                then
                    if (onechar != '~') then return 1; end if;
                    if (twochar != '~') then return -1; end if;
                    --raise notice 'passed tilde chars: %, %', onechar, twochar;
                    one := substr(one, 2);
                    two := substr(two, 2);
                    continue;
                end if;

                /*
                 * Handle caret separator. Concept is the same as tilde,
                 * except that if one of the strings ends (base version),
                 * the other is considered as higher version.
                 */
                onechar := substr(one, 1, 1);
                twochar := substr(two, 1, 1);
                --raise notice 'new chars 2: %, %', onechar, twochar;
                if (onechar = '^' or twochar = '^')
                then
                    if (one = '') then return -1; end if;
                    --raise notice 'passed caret chars 1: %, %', onechar, twochar;
                    if (two = '') then return 1; end if;
                    --raise notice 'passed caret chars 2: %, %', onechar, twochar;
                    if (onechar != '^') then return 1; end if;
                    --raise notice 'passed caret chars 3: %, %', onechar, twochar;
                    if (twochar != '^') then return -1; end if;
                    --raise notice 'passed caret chars 4: %, %', onechar, twochar;
                    one := substr(one, 2);
                    two := substr(two, 2);
                    continue;
                end if;

                if (not (one <> '' and two <> '')) then exit segment_loop; end if;

                str1 := one;
                str2 := two;
                if rpm.isdigit(str1) or rpm.isdigit(str2)
                then
                    str1 := ltrim(str1, digits);
                    str2 := ltrim(str2, digits);
                    isnum := true;
                else
                    str1 := ltrim(str1, alpha);
                    str2 := ltrim(str2, alpha);
                    isnum := false;
                end if;
                if str1 <> ''
                then segm1 := substr(one, 1, length(one) - length(str1));
                else segm1 := one;
                end if;

                if str2 <> ''
                then segm2 := substr(two, 1, length(two) - length(str2));
                else segm2 := two;
                end if;

                if isnum
                then
                    if segm1 = '' then return -1; end if;
                    if segm2 = '' then return 1; end if;

                    segm1 := ltrim(segm1, '0');
                    segm2 := ltrim(segm2, '0');

                    if segm1 = '' and segm2 <> ''
                    then
                        return -1;
                    end if;
                    if segm1 <> '' and segm2 = ''
                    then
                        return 1;
                    end if;
                    if length(segm1) < length(segm2) then return -1; end if;
                    if length(segm1) > length(segm2) then return 1; end if;
                end if;
                if segm1 < segm2 then return -1; end if;
                if segm1 > segm2 then return 1; end if;
               one := str1;
                two := str2;
            end;
        end loop segment_loop;
     
        if one = '' and two = '' then return 0; end if;
        if one = '' then return -1; end if;
        return 1;
    END ;
$$ language 'plpgsql';



   create or replace FUNCTION vercmp(
        e1 VARCHAR, v1 VARCHAR, r1 VARCHAR, 
        e2 VARCHAR, v2 VARCHAR, r2 VARCHAR)
    RETURNS INTEGER as $$
    declare
        rc INTEGER;
          ep1 INTEGER;
          ep2 INTEGER;
          BEGIN
            if e1 is null or e1 = '' then
              ep1 := 0;
            else
              ep1 := e1::integer;
            end if;
            if e2 is null or e2 = '' then
              ep2 := 0;
            else
              ep2 := e2::integer;
            end if;
            -- Epochs are non-null; compare them
            if ep1 < ep2 then return -1; end if;
            if ep1 > ep2 then return 1; end if;
            rc := rpm.rpmstrcmp(v1, v2);
            if rc != 0 then return rc; end if;
           return rpm.rpmstrcmp(r1, r2);
         END;
         $$ language 'plpgsql';

update pg_settings set setting = overlay( setting placing '' from 1 for (length('rpm')+1) ) where name = 'search_path';


-- Source: views/common/AccountsReport.sql
    
CREATE OR REPLACE VIEW AccountsReport AS
  SELECT Account.mgm_id
            , Account.organization
            , Account.account_id
            , Account.username
            , Account.last_name
            , Account.first_name
            , Account.position
            , Account.email
            , string_agg(AccountGroup.account_group_type_name, ';') AS roles
            , Account.creation_time
            , Account.last_login_time
            , Account.status
            , Account.md5_encryption
            , Account.synced_date
    FROM Account
            LEFT JOIN AccountGroup ON ( Account.mgm_id = AccountGroup.mgm_id AND Account.account_id = AccountGroup.account_id )
GROUP BY Account.mgm_id
            , Account.organization
            , Account.account_id
            , Account.username
            , Account.last_name
            , Account.first_name
            , Account.position
            , Account.email
            , Account.creation_time
            , Account.last_login_time
            , Account.status
            , Account.md5_encryption
            , Account.synced_date
ORDER BY Account.mgm_id, Account.organization, Account.account_id
;

-- Source: views/common/AccountsSystemsReport.sql
    
CREATE OR REPLACE VIEW AccountsSystemsReport AS
  WITH org_admins AS (
      SELECT mgm_id, account_id
        FROM AccountGroup
       WHERE account_group_type_label = 'org_admin'
  ), system_users AS (
      SELECT true as is_admin
              , Account.mgm_id
              , Account.account_id
              , System.system_id
              , NULL as group_name
        FROM System
              INNER JOIN Account ON ( System.mgm_id = Account.mgm_id AND System.organization = Account.organization )
    UNION
      SELECT false AS is_admin
                , SystemGroupPermission.mgm_id
                , SystemGroupPermission.account_id
                , SystemGroupMember.system_id
                , SystemGroupPermission.group_name
        FROM SystemGroupPermission
                INNER JOIN SystemGroupMember ON ( SystemGroupPermission.mgm_id = SystemGroupMember.mgm_id AND SystemGroupPermission.system_group_id = SystemGroupMember.system_group_id )
  ), users_details AS (
    SELECT Account.mgm_id
              , Account.account_id
              , Account.username
              , Account.organization
              , org_admins.account_id IS NOT NULL AS is_admin
              , Account.synced_date
      FROM Account
              LEFT JOIN org_admins ON ( Account.mgm_id = org_admins.mgm_id AND Account.account_id = org_admins.account_id )
  )
  SELECT users_details.mgm_id
            , users_details.account_id
            , users_details.username
            , users_details.organization
            , system_users.system_id
            , system_users.group_name
            , users_details.is_admin
            , users_details.synced_date
    FROM users_details
            LEFT JOIN system_users ON ( users_details.mgm_id = system_users.mgm_id AND users_details.is_admin = system_users.is_admin AND users_details.account_id = system_users.account_id)
   WHERE system_users.system_id IS NOT NULL
ORDER BY users_details.mgm_id, users_details.account_id, system_users.system_id
;

-- Source: views/common/ActionsReport.sql
    
CREATE OR REPLACE VIEW ActionsReport AS
  SELECT DISTINCT SystemAction.mgm_id
             , SystemAction.action_id
             , SystemAction.earliest_action
             , SystemAction.event
             , SystemAction.action_name
             , SystemAction.scheduler_id
             , SystemAction.scheduler_username
             , string_agg(SystemAction.hostname, ';') FILTER(WHERE status = 'Picked Up' OR status = 'Queued') OVER(PARTITION BY action_id) AS in_progress_systems
             , string_agg(SystemAction.hostname, ';') FILTER(WHERE status = 'Completed') OVER(PARTITION BY action_id) AS completed_systems
             , string_agg(SystemAction.hostname, ';') FILTER(WHERE status = 'Failed') OVER(PARTITION BY action_id) AS failed_systems
             , SystemAction.archived
             , SystemAction.synced_date
    FROM SystemAction
ORDER BY SystemAction.mgm_id, SystemAction.action_id
;

-- Source: views/common/ChannelPackagesReport.sql
    
CREATE OR REPLACE VIEW ChannelPackagesReport AS
  SELECT Channel.mgm_id
            , Channel.label AS channel_label
            , Channel.name AS channel_name
            , Package.name
            , Package.version
            , Package.release
            , Package.epoch
            , Package.arch
            , case when Package.epoch is not null then Package.epoch || ':' else '' end || Package.name || '-' || Package.version || '-' || Package.release || '.' || Package.arch AS full_package_name
            , Package.synced_date
    FROM Channel
            INNER JOIN ChannelPackage ON ( Channel.mgm_id = ChannelPackage.mgm_id AND Channel.channel_id = ChannelPackage.channel_id )
            INNER JOIN Package ON ( Channel.mgm_id = Package.mgm_id AND ChannelPackage.package_id = Package.package_id )
ORDER BY Channel.mgm_id, Channel.label, Package.name, Package.version, Package.release, Package.epoch, Package.arch
;

-- Source: views/common/ChannelsReport.sql
    
CREATE OR REPLACE VIEW ChannelsReport AS
  SELECT Channel.mgm_id
            , Channel.channel_id
            , Channel.label AS channel_label
            , Channel.name AS channel_name
            , COUNT(ChannelPackage.channel_id) AS number_of_packages
            , Channel.organization
            , Channel.synced_date
    FROM Channel
            LEFT JOIN ChannelPackage ON ( Channel.mgm_id = ChannelPackage.mgm_id AND Channel.channel_id = ChannelPackage.channel_id )
GROUP BY Channel.mgm_id, Channel.channel_id, Channel.label, Channel.name, Channel.organization, Channel.synced_date
ORDER BY Channel.mgm_id, Channel.channel_id
;

-- Source: views/common/ClonedChannelsReport.sql
    
CREATE OR REPLACE VIEW ClonedChannelsReport AS
  SELECT original.mgm_id
            , original.channel_id AS original_channel_id
            , original.label AS original_channel_label
            , original.name AS original_channel_name
            , cloned.channel_id AS new_channel_id
            , cloned.label AS new_channel_label
            , cloned.name AS new_channel_name
            , cloned.synced_date
    FROM Channel original
            INNER JOIN Channel cloned ON ( cloned.mgm_id = original.mgm_id AND cloned.original_channel_id = original.channel_id )
ORDER BY original.mgm_id, original.channel_id
;

-- Source: views/common/CustomChannelsReport.sql
    
CREATE OR REPLACE VIEW CustomChannelsReport AS
  WITH repositories AS (
      SELECT mgm_id, channel_id, string_agg(repository_id || ' - ' || repository_label, ';')  AS channel_repositories
        FROM ChannelRepository
    GROUP BY mgm_id, channel_id
  )
  SELECT Channel.mgm_id
             , Channel.organization
             , Channel.channel_id
             , Channel.label
             , Channel.name
             , Channel.summary
             , Channel.description
             , Channel.parent_channel_label
             , Channel.arch
             , Channel.checksum_type
             , repositories.channel_repositories
             , Channel.synced_date
    FROM Channel
             LEFT JOIN repositories ON ( Channel.mgm_id = repositories.mgm_id AND Channel.channel_id = repositories.channel_id )
   WHERE Channel.organization IS NOT NULL
ORDER BY Channel.mgm_id, Channel.organization, Channel.channel_id
;

-- Source: views/common/CustomInfoReport.sql
    
CREATE OR REPLACE VIEW CustomInfoReport AS
  SELECT SystemCustomInfo.mgm_id
            , SystemCustomInfo.system_id
            , System.profile_name AS system_name
            , SystemCustomInfo.organization
            , SystemCustomInfo.key
            , SystemCustomInfo.value
            , SystemCustomInfo.synced_date
    FROM SystemCustomInfo
            INNER JOIN System ON (SystemCustomInfo.mgm_id = System.mgm_id AND SystemCustomInfo.system_id = System.system_id )
ORDER BY SystemCustomInfo.mgm_id, SystemCustomInfo.organization, SystemCustomInfo.system_id, SystemCustomInfo.key
;

-- Source: views/common/ErrataChannelsReport.sql
    
CREATE OR REPLACE VIEW ErrataChannelsReport AS
  SELECT mgm_id
            , advisory_name
            , errata_id
            , channel_label
            , channel_id
            , synced_date
    FROM ChannelErrata
ORDER BY mgm_id, advisory_name, errata_id, channel_label, channel_id
;

-- Source: views/common/ErrataListReport.sql
    
CREATE OR REPLACE VIEW ErrataListReport AS
  SELECT Errata.mgm_id
            , Errata.errata_id
            , Errata.advisory_name
            , Errata.advisory_type
            , Errata.cve
            , Errata.synopsis
            , Errata.issue_date
            , Errata.update_date
            , COUNT(SystemErrata.system_id) AS affected_systems
            , Errata.synced_date
    FROM Errata
            LEFT JOIN SystemErrata ON ( Errata.mgm_id = SystemErrata.mgm_id AND Errata.errata_id = SystemErrata.errata_id )
GROUP BY Errata.mgm_id
            , Errata.errata_id
            , Errata.advisory_name
            , Errata.advisory_type
            , Errata.cve
            , Errata.synopsis
            , Errata.issue_date
            , Errata.update_date
            , Errata.synced_date
ORDER BY Errata.mgm_id, Errata.advisory_name
;

-- Source: views/common/ErrataSystemsReport.sql
    
CREATE OR REPLACE VIEW ErrataSystemsReport AS
  WITH V6Addresses AS (
          SELECT mgm_id, system_id, interface_id, string_agg(address || ' (' || scope || ')', ';') AS ip6_addresses
            FROM SystemNetAddressV6
        GROUP BY mgm_id, system_id, interface_id
  )
  SELECT SystemErrata.mgm_id
              , SystemErrata.errata_id
              , SystemErrata.advisory_name
              , SystemErrata.system_id
              , System.profile_name
              , System.hostname
              , SystemNetAddressV4.address AS ip_address
              , V6Addresses.ip6_addresses
              , SystemErrata.synced_date
    FROM SystemErrata
            INNER JOIN System ON ( SystemErrata.mgm_id = System.mgm_id AND SystemErrata.system_id = System.system_id )
            LEFT JOIN SystemNetInterface ON ( System.mgm_id = SystemNetInterface.mgm_id AND System.system_id = SystemNetInterface.system_id AND primary_interface )
            LEFT JOIN SystemNetAddressV4 ON ( System.mgm_id = SystemNetAddressV4.mgm_id AND System.system_id = SystemNetAddressV4.system_id AND SystemNetInterface.interface_id = SystemNetAddressV4.interface_id )
            LEFT JOIN V6Addresses ON ( System.mgm_id = V6Addresses.mgm_id AND System.system_id = V6Addresses.system_id AND SystemNetInterface.interface_id = V6Addresses.interface_id )
ORDER BY SystemErrata.mgm_id, SystemErrata.errata_id, SystemErrata.system_id
;

-- Source: views/common/HistoryReport.sql
    
CREATE OR REPLACE VIEW HistoryReport AS
      SELECT mgm_id
                , system_id
                , action_id AS event_id
                , hostname
                , event
                , completion_time AS event_time
                , status
                , event_data
                , synced_date
        FROM SystemAction

    UNION ALL

      SELECT mgm_id
                , system_id
                , history_id AS event_id
                , hostname
                , event
                , event_time
                , 'Done' AS status
                , event_data
                , synced_date
        FROM SystemHistory
;

-- Source: views/common/HostGuestsReport.sql
    
CREATE OR REPLACE VIEW HostGuestsReport AS
  SELECT mgm_id
            , host_system_id AS host
            , virtual_system_id AS guest
            , synced_date
    FROM SystemVirtualData
   WHERE host_system_id IS NOT NULL
            AND virtual_system_id IS NOT NULL
ORDER BY mgm_id, host_system_id, virtual_system_id
;

-- Source: views/common/InventoryReport.sql
    
CREATE OR REPLACE VIEW InventoryReport AS
    -- CTEs to group all one to many relationship joining values with ; as separator
    WITH Entitlements AS (
        SELECT mgm_id, system_id, string_agg(system_group_id || ' - ' || name, ';') AS entitlements
          FROM systementitlement
      GROUP BY mgm_id, system_id
    ), Groups AS (
        SELECT mgm_id, system_id, string_agg(system_group_id || ' - ' || group_name, ';') AS system_groups
          FROM SystemGroupMember
      GROUP BY mgm_id, system_id
    ), ConfigChannels AS (
        SELECT mgm_id, system_id, string_agg(config_channel_id || ' - ' || name, ';') AS configuration_channels
          FROM SystemConfigChannel
      GROUP BY mgm_id, system_id
    ), Channels AS (
        SELECT mgm_id, system_id, string_agg(channel_id || ' - ' || name, ';') AS software_channels
          FROM SystemChannel
      GROUP BY mgm_id, system_id
    ), V6Addresses AS (
        SELECT mgm_id, system_id, interface_id, string_agg(address || ' (' || scope || ')', ';') AS ip6_addresses
          FROM SystemNetAddressV6
      GROUP BY mgm_id, system_id, interface_id
    )
    SELECT System.mgm_id
              , System.system_id
              , System.profile_name
              , System.hostname
              , System.minion_id
              , System.machine_id
              , System.registered_by
              , System.registration_time
              , System.last_checkin_time
              , System.kernel_version
              , System.organization
              , System.architecture
              , System.hardware
              , SystemNetInterface.name  AS primary_interface
              , SystemNetInterface.hardware_address AS hardware_address
              , SystemNetAddressV4.address AS ip_address
              , V6Addresses.ip6_addresses
              , ConfigChannels.configuration_channels
              , Entitlements.entitlements
              , Groups.system_groups
              , SystemVirtualdata.host_system_id AS virtual_host
              , SystemVirtualdata.virtual_system_id IS NULL AS is_virtualized
              , SystemVirtualdata.instance_type_name AS virt_type
              , Channels.software_channels
              , COALESCE(SystemOutdated.packages_out_of_date, (0)::bigint) AS packages_out_of_date
              , COALESCE(SystemOutdated.errata_out_of_date, (0)::bigint) AS errata_out_of_date
              , System.synced_date
      FROM System
              LEFT JOIN SystemVirtualdata ON ( System.mgm_id = SystemVirtualdata.mgm_id AND System.system_id = SystemVirtualdata.virtual_system_id )
              LEFT JOIN SystemOutdated ON ( System.mgm_id = SystemOutdated.mgm_id AND System.system_id = SystemOutdated.system_id )
              LEFT JOIN SystemNetInterface ON (System.mgm_id = SystemNetInterface.mgm_id AND System.system_id = SystemNetInterface.system_id AND SystemNetInterface.primary_interface)
              LEFT JOIN SystemNetAddressV4 ON (System.mgm_id = SystemNetAddressV4.mgm_id AND System.system_id = SystemNetAddressV4.system_id AND SystemNetInterface.interface_id = SystemNetAddressV4.interface_id)
              LEFT JOIN V6Addresses ON (System.mgm_id = V6Addresses.mgm_id AND System.system_id = V6Addresses.system_id AND SystemNetInterface.interface_id = V6Addresses.interface_id)
              LEFT JOIN Entitlements ON ( System.mgm_id = entitlements.mgm_id AND System.system_id = entitlements.system_id )
              LEFT JOIN Groups ON ( System.mgm_id = Groups.mgm_id AND System.system_id = Groups.system_id )
              LEFT JOIN ConfigChannels ON ( System.mgm_id = ConfigChannels.mgm_id AND System.system_id = ConfigChannels.system_id )
              LEFT JOIN Channels ON ( System.mgm_id = Channels.mgm_id AND System.system_id = Channels.system_id )
  ORDER BY System.mgm_id, System.system_id
;

-- Source: views/common/PackagesUpdatesAllReport.sql
    
CREATE OR REPLACE VIEW PackagesUpdatesAllReport AS
  SELECT System.mgm_id
            , System.system_id
            , System.organization
            , SystemPackageUpdate.name AS package_name
            , SystemPackageInstalled.epoch AS package_epoch
            , SystemPackageInstalled.version AS package_version
            , SystemPackageInstalled.release AS package_release
            , SystemPackageInstalled.arch AS package_arch
            , SystemPackageUpdate.epoch AS newer_epoch
            , SystemPackageUpdate.version AS newer_version
            , SystemPackageUpdate.release AS newer_release
            , SystemPackageUpdate.synced_date
    FROM System
            INNER JOIN SystemPackageUpdate ON ( System.mgm_id = SystemPackageUpdate.mgm_id AND System.system_id = SystemPackageUpdate.system_id )
            INNER JOIN SystemPackageInstalled ON ( System.mgm_id = SystemPackageInstalled.mgm_id AND System.system_id = SystemPackageInstalled.system_id AND SystemPackageInstalled.name = SystemPackageUpdate.name )
ORDER BY System.mgm_id, System.organization, System.system_id, SystemPackageUpdate.name
;

-- Source: views/common/PackagesUpdatesNewestReport.sql
    
CREATE OR REPLACE VIEW PackagesUpdatesNewestReport AS
  SELECT System.mgm_id
            , System.system_id
            , System.organization
            , SystemPackageUpdate.name AS package_name
            , SystemPackageInstalled.epoch AS package_epoch
            , SystemPackageInstalled.version AS package_version
            , SystemPackageInstalled.release AS package_release
            , SystemPackageInstalled.arch AS package_arch
            , SystemPackageUpdate.epoch AS newer_epoch
            , SystemPackageUpdate.version AS newer_version
            , SystemPackageUpdate.release AS newer_release
            , SystemPackageUpdate.synced_date
    FROM System
            INNER JOIN SystemPackageUpdate ON ( System.mgm_id = SystemPackageUpdate.mgm_id AND System.system_id = SystemPackageUpdate.system_id )
            INNER JOIN SystemPackageInstalled ON ( System.mgm_id = SystemPackageInstalled.mgm_id AND System.system_id = SystemPackageInstalled.system_id AND SystemPackageInstalled.name = SystemPackageUpdate.name )
   WHERE SystemPackageUpdate.is_latest
ORDER BY System.mgm_id, System.organization, System.system_id, SystemPackageUpdate.name
;

-- Source: views/common/ProxyOverviewReport.sql
    
CREATE OR REPLACE VIEW ProxyOverviewReport AS
    SELECT prx.mgm_id
              , prx.system_id AS proxy_id
              , prx.hostname AS proxy_name
              , sys.hostname AS system_name
              , sys.system_id
              , prx.synced_date
      FROM system prx
              INNER JOIN system sys ON sys.proxy_system_id = prx.system_id AND sys.mgm_id = prx.mgm_id
    WHERE prx.is_proxy
 ORDER BY prx.mgm_id, prx.system_id, sys.system_id
;

-- Source: views/common/ScapScanReport.sql
    
CREATE OR REPLACE VIEW ScapScanReport AS
  SELECT XccdScan.mgm_id
            , XccdScan.scan_id
            , System.system_id
            , XccdScan.action_id
            , System.hostname
            , System.organization
            , SystemNetAddressV4.address AS ip_address
            , XccdScan.name
            , XccdScan.benchmark
            , XccdScan.benchmark_version
            , XccdScan.profile
            , XccdScan.profile_title
            , XccdScan.end_time
            , XccdScan.pass
            , XccdScan.fail
            , XccdScan.error
            , XccdScan.not_selected
            , XccdScan.informational
            , XccdScan.other
            , XccdScan.synced_date
    FROM XccdScan
            LEFT JOIN System ON ( XccdScan.mgm_id = System.mgm_id AND XccdScan.system_id = System.system_id )
            LEFT JOIN SystemNetInterface ON (System.mgm_id = SystemNetInterface.mgm_id AND System.system_id = SystemNetInterface.system_id AND SystemNetInterface.primary_interface)
            LEFT JOIN SystemNetAddressV4 ON (System.mgm_id = SystemNetAddressV4.mgm_id AND System.system_id = SystemNetAddressV4.system_id AND SystemNetInterface.interface_id = SystemNetAddressV4.interface_id)
ORDER BY XccdScan.mgm_id
            , XccdScan.scan_id
            , System.system_id
            , XccdScan.end_time
;

-- Source: views/common/ScapScanResultReport.sql
    
CREATE OR REPLACE VIEW ScapScanResultReport AS
  SELECT XccdScanResult.mgm_id
            , XccdScanResult.scan_id
            , XccdScanResult.rule_id
            , XccdScanResult.idref
            , XccdScanResult.rulesystem
            , System.system_id
            , System.hostname
            , System.organization
            , XccdScanResult.ident
            , XccdScanResult.result
            , XccdScanResult.synced_date
    FROM XccdScanResult
            LEFT JOIN System ON ( XccdScanResult.mgm_id = System.mgm_id AND XccdScanResult.system_id = System.system_id )
ORDER BY mgm_id, scan_id, rule_id
;

-- Source: views/common/SystemExtraPackagesReport.sql
    
CREATE OR REPLACE VIEW SystemExtraPackagesReport AS
  WITH packages_from_channels AS (
    SELECT SystemPackageInstalled.mgm_id
              , SystemPackageInstalled.system_id
              , Package.package_id
              , SystemPackageInstalled.name
              , SystemPackageInstalled.epoch
              , SystemPackageInstalled.version
              , SystemPackageInstalled.release
              , SystemPackageInstalled.arch
              , SystemPackageInstalled.type
    FROM SystemPackageInstalled
              INNER JOIN SystemChannel ON ( SystemPackageInstalled.mgm_id = SystemChannel.mgm_id AND SystemPackageInstalled.system_id = SystemChannel.system_id )
              INNER JOIN ChannelPackage ON ( SystemChannel.mgm_id = ChannelPackage.mgm_id AND ChannelPackage.channel_id = SystemChannel.channel_id)
              INNER JOIN Package ON (  SystemPackageInstalled.mgm_id = Package.mgm_id
                                                AND ChannelPackage.package_id  = Package.package_id 
                                                AND Package.name = SystemPackageInstalled.name
                                                AND COALESCE(Package.epoch, '') = COALESCE(SystemPackageInstalled.epoch, '')
                                                AND Package.version = SystemPackageInstalled.version
                                                AND Package.release = SystemPackageInstalled.release
                                                AND Package.arch = SystemPackageInstalled.arch
                                           )
  )
  SELECT System.mgm_id
             , System.system_id
             , System.hostname AS system_name
             , System.organization
             , SystemPackageInstalled.name AS package_name
             , SystemPackageInstalled.epoch AS package_epoch
             , SystemPackageInstalled.version AS package_version
             , SystemPackageInstalled.release AS package_release
             , SystemPackageInstalled.arch AS package_arch
             , SystemPackageInstalled.synced_date
    FROM System
           INNER JOIN SystemPackageInstalled ON ( System.mgm_id = SystemPackageInstalled.mgm_id AND System.system_id = SystemPackageInstalled.system_id )
           LEFT JOIN packages_from_channels ON ( SystemPackageInstalled.mgm_id = packages_from_channels.mgm_id AND SystemPackageInstalled.system_id = packages_from_channels.system_id
                                                     AND SystemPackageInstalled.name = packages_from_channels.name
                                                     AND COALESCE(SystemPackageInstalled.epoch, '') = COALESCE(packages_from_channels.epoch, '')
                                                     AND SystemPackageInstalled.version = packages_from_channels.version
                                                     AND SystemPackageInstalled.release = packages_from_channels.release
                                                     AND SystemPackageInstalled.arch = packages_from_channels.arch
                                               )
   WHERE packages_from_channels.package_id IS NULL
ORDER BY System.mgm_id, System.organization, System.system_id, SystemPackageInstalled.name
;

-- Source: views/common/SystemGroupsReport.sql
    
CREATE OR REPLACE VIEW SystemGroupsReport AS
  SELECT mgm_id
            , system_group_id
            , name
            , current_members
            , organization
            , synced_date
    FROM SystemGroup
ORDER BY mgm_id, system_group_id
;

-- Source: views/common/SystemGroupsSystemsReport.sql
    
CREATE OR REPLACE VIEW SystemGroupsSystemsReport AS
   SELECT SystemGroupMember.mgm_id
              , SystemGroupMember.system_group_id AS group_id
              , SystemGroupMember.group_name AS group_name
              , System.system_id
              , System.profile_name AS system_name
              , SystemGroupMember.synced_date
     FROM SystemGroupMember
              INNER JOIN System ON ( SystemGroupMember.mgm_id = System.mgm_id AND SystemGroupMember.system_id = System.system_id )
 ORDER BY SystemGroupMember.mgm_id, SystemGroupMember.system_group_id, SystemGroupMember.system_id
;

-- Source: views/common/SystemHistoryAutoinstallationReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryAutoinstallationReport AS
  SELECT mgm_id
            , system_id
            , action_id
            , earliest_action
            , pickup_time
            , completion_time
            , status
            , event
            , event_data
            , synced_date
    FROM SystemAction
   WHERE event IN ( 'Schedule a package sync for auto installations', 'Initiate an auto installation' )
ORDER BY mgm_id, system_id, action_id
;

-- Source: views/common/SystemHistoryChannelsReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryChannelsReport AS
  SELECT mgm_id
              , system_id
              , history_id
              , event_time
              , 'Done' AS status
              , event
              , event_data
              , synced_date
    FROM SystemHistory
   WHERE event IN ('Subscribed to channel', 'Unsubscribed from channel')
ORDER BY mgm_id, system_id, history_id
;

-- Source: views/common/SystemHistoryConfigurationReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryConfigurationReport AS
  SELECT mgm_id
            , system_id
            , action_id
            , earliest_action
            , pickup_time
            , completion_time
            , status
            , event
            , event_data
            , synced_date
    FROM SystemAction
   WHERE event IN ( 'Upload config file data to server',
                    'Verify deployed config files',
                    'Show differences between profiled config files and deployed config files',
                    'Upload config file data based upon mtime to server',
                    'Deploy config files to system'
                  )
ORDER BY mgm_id, system_id, action_id
;

-- Source: views/common/SystemHistoryEntitlementsReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryEntitlementsReport AS
  SELECT mgm_id
              , system_id
              , history_id
              , event_time
              , 'Done' AS status
              , event
              , event_data
              , synced_date
    FROM SystemHistory
   WHERE event NOT IN ('Subscribed to channel', 'Unsubscribed from channel')
ORDER BY mgm_id, system_id, history_id
;

-- Source: views/common/SystemHistoryErrataReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryErrataReport AS
  SELECT mgm_id
            , system_id
            , action_id
            , earliest_action
            , pickup_time
            , completion_time
            , status
            , event
            , event_data
            , synced_date
    FROM SystemAction
   WHERE event = 'Patch Update'
ORDER BY mgm_id, system_id, action_id
;

-- Source: views/common/SystemHistoryPackagesReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryPackagesReport AS
  SELECT mgm_id
            , system_id
            , action_id
            , earliest_action
            , pickup_time
            , completion_time
            , status
            , event
            , event_data
            , synced_date
    FROM SystemAction
   WHERE event IN ( 'Package Upgrade', 'Package Removal' )
ORDER BY mgm_id, system_id, action_id
;

-- Source: views/common/SystemHistoryScapReport.sql
    
CREATE OR REPLACE VIEW SystemHistoryScapReport AS
  SELECT mgm_id
            , system_id
            , action_id
            , earliest_action
            , pickup_time
            , completion_time
            , status
            , event
            , event_data
            , synced_date
    FROM SystemAction
   WHERE event = 'OpenSCAP xccdf scanning'
ORDER BY mgm_id, system_id, action_id
;

-- Source: views/common/SystemInactivityReport.sql
    
CREATE OR REPLACE VIEW SystemInactivityReport AS
  SELECT mgm_id
            , system_id
            , profile_name AS system_name
            , organization
            , last_checkin_time
            , (synced_date - last_checkin_time) AS inactivity
            , synced_date
    FROM system
ORDER BY mgm_id, system_id, organization
;

-- Source: views/common/SystemPackagesInstalledReport.sql
    
CREATE OR REPLACE VIEW SystemPackagesInstalledReport AS
  SELECT SystemPackageInstalled.mgm_id
            , SystemPackageInstalled.system_id
            , System.organization
            , SystemPackageInstalled.name AS package_name
            , SystemPackageInstalled.epoch AS package_epoch
            , SystemPackageInstalled.version AS package_version
            , SystemPackageInstalled.release AS package_release
            , SystemPackageInstalled.arch AS package_arch
            , SystemPackageInstalled.synced_date
    FROM SystemPackageInstalled
             INNER JOIN System ON System.system_id = SystemPackageInstalled.system_id
ORDER BY SystemPackageInstalled.mgm_id, SystemPackageInstalled.system_id, SystemPackageInstalled.name
;

-- Source: data/common/VersionInfo.sql
    
DELETE
  FROM VersionInfo
 WHERE label = 'schema' and name = 'uyuni-reportdb-schema';

INSERT
  INTO VersionInfo ( name, label, version, release )
VALUES ('uyuni-reportdb-schema', 'schema', '4.4.4', '1');

COMMIT;


-- Source: end.sql
    
create or replace function create_varnull_constriants() returns integer as $$
declare
    tabs record;
    total integer default 0;
begin

    for tabs in select 
        c.relname as "tab",
        a.attname as "col"
    from
        pg_catalog.pg_attribute a
        left outer join pg_catalog.pg_class c on a.attrelid = c.oid 
    where
        -- skip system columns
        a.attnum > 0
        -- skip dropped columns
        and not a.attisdropped
        -- filter only varchars
        and a.atttypid = 1043
        -- skip cols that already has this constraint
        and not exists (
            select 1 from pg_catalog.pg_constraint 
            where conname = 'vn_' || c.relname || '_' || a.attname
        )
        -- filter only tables owned by current user
        and a.attrelid in (
            select c.oid from pg_catalog.pg_class c
            where relkind = 'r' and pg_catalog.pg_table_is_visible(c.oid) and relowner = (
                select oid from pg_catalog.pg_roles where rolname = current_user
            )
        ) loop
        begin
            -- create constraint
            execute 'alter table ' || tabs.tab || ' add constraint vn_' ||
                tabs.tab || '_' || tabs.col || ' check (' || tabs.col || ' <> '''')';
            -- count them
        exception when others then
            total = total + 1;
            raise warning '% unable to create constraint for %.%', now(), tabs.tab, tabs.col;
        end;
    end loop;

    return total;
end;
$$ language plpgsql;

select create_varnull_constriants();


